@@ -1,12 +1,17 @@
-from collections import OrderedDict
-
 import numpy as np
 import pytest
 
-import pandas.util._test_decorators as td
-
-import pandas as pd
-from pandas import DataFrame, Index, Series, Timestamp, compat, concat
+from pandas import (
+    DataFrame,
+    Index,
+    MultiIndex,
+    Period,
+    Series,
+    Timestamp,
+    concat,
+    date_range,
+    timedelta_range,
+)
 import pandas._testing as tm
 from pandas.core.base import SpecificationError
 
@@ -82,7 +87,7 @@ def test_agg():
 
     result = r.aggregate([np.mean, np.std])
     expected = concat([a_mean, a_std, b_mean, b_std], axis=1)
-    expected.columns = pd.MultiIndex.from_product([["A", "B"], ["mean", "std"]])
+    expected.columns = MultiIndex.from_product([["A", "B"], ["mean", "std"]])
     tm.assert_frame_equal(result, expected)
 
     result = r.aggregate({"A": np.mean, "B": np.std})
@@ -92,7 +97,7 @@ def test_agg():
 
     result = r.aggregate({"A": ["mean", "std"]})
     expected = concat([a_mean, a_std], axis=1)
-    expected.columns = pd.MultiIndex.from_tuples([("A", "mean"), ("A", "std")])
+    expected.columns = MultiIndex.from_tuples([("A", "mean"), ("A", "std")])
     tm.assert_frame_equal(result, expected)
 
     result = r["A"].aggregate(["mean", "sum"])
@@ -114,7 +119,7 @@ def test_agg():
     expected = concat([a_mean, a_std, b_mean, b_std], axis=1)
 
     exp_cols = [("A", "mean"), ("A", "std"), ("B", "mean"), ("B", "std")]
-    expected.columns = pd.MultiIndex.from_tuples(exp_cols)
+    expected.columns = MultiIndex.from_tuples(exp_cols)
     tm.assert_frame_equal(result, expected, check_like=True)
 
 
@@ -138,7 +143,7 @@ def test_agg_consistency():
     r = df.rolling(window=3)
 
     result = r.agg([np.sum, np.mean]).columns
-    expected = pd.MultiIndex.from_product([list("AB"), ["sum", "mean"]])
+    expected = MultiIndex.from_product([list("AB"), ["sum", "mean"]])
     tm.assert_index_equal(result, expected)
 
     result = r["A"].agg([np.sum, np.mean]).columns
@@ -146,7 +151,7 @@ def test_agg_consistency():
     tm.assert_index_equal(result, expected)
 
     result = r.agg({"A": [np.sum, np.mean]}).columns
-    expected = pd.MultiIndex.from_tuples([("A", "sum"), ("A", "mean")])
+    expected = MultiIndex.from_tuples([("A", "sum"), ("A", "mean")])
     tm.assert_index_equal(result, expected)
 
 
@@ -163,7 +168,7 @@ def test_agg_nested_dicts():
     expected = concat(
         [r["A"].mean(), r["A"].std(), r["B"].mean(), r["B"].std()], axis=1
     )
-    expected.columns = pd.MultiIndex.from_tuples(
+    expected.columns = MultiIndex.from_tuples(
         [("ra", "mean"), ("ra", "std"), ("rb", "mean"), ("rb", "std")]
     )
     with pytest.raises(SpecificationError, match=msg):
@@ -195,21 +200,21 @@ def test_count_nonnumeric_types():
             "int": [1, 2, 3],
             "float": [4.0, 5.0, 6.0],
             "string": list("abc"),
-            "datetime": pd.date_range("20170101", periods=3),
-            "timedelta": pd.timedelta_range("1 s", periods=3, freq="s"),
+            "datetime": date_range("20170101", periods=3),
+            "timedelta": timedelta_range("1 s", periods=3, freq="s"),
             "periods": [
-                pd.Period("2012-01"),
-                pd.Period("2012-02"),
-                pd.Period("2012-03"),
+                Period("2012-01"),
+                Period("2012-02"),
+                Period("2012-03"),
             ],
             "fl_inf": [1.0, 2.0, np.Inf],
             "fl_nan": [1.0, 2.0, np.NaN],
             "str_nan": ["aa", "bb", np.NaN],
             "dt_nat": dt_nat_col,
             "periods_nat": [
-                pd.Period("2012-01"),
-                pd.Period("2012-02"),
-                pd.Period(None),
+                Period("2012-01"),
+                Period("2012-02"),
+                Period(None),
             ],
         },
         columns=cols,
@@ -240,30 +245,6 @@ def test_count_nonnumeric_types():
     tm.assert_frame_equal(result, expected)
 
 
-@td.skip_if_no_scipy
-@pytest.mark.filterwarnings("ignore:can't resolve:ImportWarning")
-def test_window_with_args():
-    # make sure that we are aggregating window functions correctly with arg
-    r = Series(np.random.randn(100)).rolling(
-        window=10, min_periods=1, win_type="gaussian"
-    )
-    expected = concat([r.mean(std=10), r.mean(std=0.01)], axis=1)
-    expected.columns = ["<lambda>", "<lambda>"]
-    result = r.aggregate([lambda x: x.mean(std=10), lambda x: x.mean(std=0.01)])
-    tm.assert_frame_equal(result, expected)
-
-    def a(x):
-        return x.mean(std=10)
-
-    def b(x):
-        return x.mean(std=0.01)
-
-    expected = concat([r.mean(std=10), r.mean(std=0.01)], axis=1)
-    expected.columns = ["a", "b"]
-    result = r.aggregate([a, b])
-    tm.assert_frame_equal(result, expected)
-
-
 def test_preserve_metadata():
     # GH 10565
     s = Series(np.arange(100), name="foo")
@@ -277,7 +258,7 @@ def test_preserve_metadata():
 @pytest.mark.parametrize(
     "func,window_size,expected_vals",
     [
-        pytest.param(
+        (
             "rolling",
             2,
             [
@@ -289,7 +270,6 @@ def test_preserve_metadata():
                 [35.0, 40.0, 60.0, 40.0],
                 [60.0, 80.0, 85.0, 80],
             ],
-            marks=pytest.mark.xfail(not compat.IS64, reason="GH-35294"),
         ),
         (
             "expanding",
@@ -308,7 +288,7 @@ def test_preserve_metadata():
 )
 def test_multiple_agg_funcs(func, window_size, expected_vals):
     # GH 15072
-    df = pd.DataFrame(
+    df = DataFrame(
         [
             ["A", 10, 20],
             ["A", 20, 30],
@@ -327,17 +307,15 @@ def test_multiple_agg_funcs(func, window_size, expected_vals):
     else:
         window = f()
 
-    index = pd.MultiIndex.from_tuples(
+    index = MultiIndex.from_tuples(
         [("A", 0), ("A", 1), ("A", 2), ("B", 3), ("B", 4), ("B", 5), ("B", 6)],
         names=["stock", None],
     )
-    columns = pd.MultiIndex.from_tuples(
+    columns = MultiIndex.from_tuples(
         [("low", "mean"), ("low", "max"), ("high", "mean"), ("high", "min")]
     )
-    expected = pd.DataFrame(expected_vals, index=index, columns=columns)
+    expected = DataFrame(expected_vals, index=index, columns=columns)
 
-    result = window.agg(
-        OrderedDict((("low", ["mean", "max"]), ("high", ["mean", "min"])))
-    )
+    result = window.agg({"low": ["mean", "max"], "high": ["mean", "min"]})
 
     tm.assert_frame_equal(result, expected)
